#include "tclInt.h"
#include "tclPort.h"

#include <lib$routines.h>
#include <clidef.h>
#include <unixlib.h>
#include <unixio.h>

int
Tcl_VmsExec(dummy, interp, argc, argv)
    ClientData dummy;			/* Not used. */
    Tcl_Interp *interp;			/* Current interpreter. */
    int argc;				/* Number of arguments. */
    char **argv;			/* Argument strings. */
{
    char *cmd;
    char buffer[256];
    char outname[256];
    int status, i, flags;
    int keepNewline, firstWord, background, result;
    unsigned int pid;
    struct {
	int l;
	char *a;
    } inDesc, cmdDesc, outDesc;
    FILE *out;

    /*
     * Check for a leading "-keepnewline" argument.
     */

    keepNewline = 0;
    for (firstWord = 1; (firstWord < argc) && (argv[firstWord][0] == '-');
	  firstWord++) {
	if (strcmp(argv[firstWord], "-keepnewline") == 0) {
	    keepNewline = 1;
	} else if (strcmp(argv[firstWord], "--") == 0) {
	    firstWord++;
	    break;
	} else {
	    Tcl_AppendResult(interp, "bad switch \"", argv[firstWord],
		    "\": must be -keepnewline or --", (char *) NULL);
	    return TCL_ERROR;
	}
    }

    if (argc <= firstWord) {
	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
		" ?switches? arg ?arg ...?\"", (char *) NULL);
	return TCL_ERROR;
    }

    /*
     * See if the command is to be run in background.
     */

    background = 0;
    if ((argv[argc-1][0] == '&') && (argv[argc-1][1] == 0)) {
	argc--;
	argv[argc] = NULL;
        background = 1;
    }

    flags = background ? CLI$M_NOWAIT : 0;

    cmd = Tcl_Concat(argc-1, argv+1);
    cmdDesc.l = strlen(cmd);
    cmdDesc.a = cmd;

    /*
	If it points to SYS$INPUT, lib$spawn fails because
	SYS$INPUT has an associated mailbox attached.

	Anyone know how to fix this?
    */
    inDesc.l = strlen("NL:");
    inDesc.a = "NL:";

    strcpy(outname, "sys$scratch:tclout.XXXXXX.");
    mktemp(outname);
    outDesc.l = strlen(outname);
    outDesc.a = outname;
    result = 1;
    status = lib$spawn(&cmdDesc, &inDesc, &outDesc, &flags, 0, &pid, &result);
    if ((status & 1) && ((status = result) & 1)) {
	if (background) {
	    char pidString[32];

	    sprintf(pidString, "%x", pid);
	    Tcl_SetResult(interp, pidString, TCL_VOLATILE);
	    result = TCL_OK;
	}
	else {
	    if (out = fopen(outname, "r")) {
		while (fgets(buffer, sizeof(buffer), out))
		    Tcl_AppendResult(interp, buffer, (char *) NULL);
		fclose(out);
		result = TCL_OK;
	    }
	    else {
		Tcl_AppendResult(interp, "error opening output file: ",
				 Tcl_PosixError(interp), (char *) NULL);
		result = TCL_ERROR;
	    }
	}
    }
    else {
	errno = EVMSERR;
	vaxc$errno = status;
	Tcl_ResetResult(interp);
	Tcl_AppendResult(interp, "error running command: ",
			 Tcl_PosixError(interp), (char *) NULL);
	result = TCL_ERROR;
    }
    delete(outname);
    ckfree(cmd);
    return result;
}
